/*
 *  Hello World Triangle Version 5 with comments
 */
#include <stdio.h>
#include <stdarg.h>
#include <math.h>
//  OpenGL prototypes with extensions
#define GL_GLEXT_PROTOTYPES
#include <GL/glut.h>

//  Global variables
double th=0;  //  Rotation angle
double zh=0;  //  Spin angle

/*
 *  printf for OpenGL
 */
#define LEN 8192  //  Print output is limited to 8192 characters
void Print(const char* format , ...)
{
   char    buf[LEN];
   char*   ch=buf;
   //  Convert arguments to a string stored in buf
   va_list args;
   va_start(args,format);
   vsnprintf(buf,LEN,format,args);
   va_end(args);
   //  Copy string to screen from current raster position
   while (*ch)
      glutBitmapCharacter(GLUT_BITMAP_HELVETICA_18,*ch++);
}

/*
 *  Draw the scene
 */
void display()
{
   //  Clear display
   glClear(GL_COLOR_BUFFER_BIT|GL_DEPTH_BUFFER_BIT);
   //  Reset transformation
   glLoadIdentity();
   //  Draw RGB triangle
   glBegin(GL_POLYGON);
   glColor3f(1.0,0.0,0.0); glVertex2f( 0.0, 0.5);
   glColor3f(0.0,1.0,0.0); glVertex2f( 0.5,-0.5);
   glColor3f(0.0,0.0,1.0); glVertex2f(-0.5,-0.5);
   glEnd();
   //  Offset and rotate second triangle
   glTranslatef(0.2,0.2,0.2);
   glRotatef(th,0.0,1.0,0.0);
   glRotatef(zh,0.0,0.0,1.0);
   //  Draw CMY triangle
   glBegin(GL_POLYGON);
   glColor3f(1.0,1.0,0.0); glVertex2f( 0.0, 0.5);
   glColor3f(0.0,1.0,1.0); glVertex2f( 0.5,-0.5);
   glColor3f(1.0,0.0,1.0); glVertex2f(-0.5,-0.5);
   glEnd();
   //  Display angle in white at lower left
   glColor3f(1,1,1);
   glWindowPos2i(5,5);
   Print("Angle=%.1f",th);
   //  Flush buffer to screen
   glFlush();
   glutSwapBuffers();
}

/*
 *  Process cursor keys
 */
void special(int key,int x,int y)
{
   //  Rotate +5 degrees on right arrow
   if (key == GLUT_KEY_RIGHT)
      th += 5;
   //  Rotate -5 degrees on right arrow
   else if (key == GLUT_KEY_LEFT)
      th -= 5;
   //  Request display update
   glutPostRedisplay();
}

/*
 *  Update projection when window is resized
 */
void reshape(int width,int height)
{
   //  Width to height ratio
   double w2h = (height>0) ? (double)width/height : 1;
   //  Viewport is entire window
   glViewport(0,0, width,height);
   //  Set projection to orthogonal unit cube with left and right adjusted to aspect ratio of window
   glMatrixMode(GL_PROJECTION);
   glLoadIdentity();
   glOrtho(-w2h,+w2h, -1.0,+1.0, -1.0,+1.0);
   //  Model view transform is identity
   glMatrixMode(GL_MODELVIEW);
   glLoadIdentity();
}

/*
 *  Function called when idle for animation
 */
void idle()
{
   //  Get elapsed time since start of run in seconds
   double t = glutGet(GLUT_ELAPSED_TIME)/1000.0;
   //  Spin angle increases 90 degrees per second
   zh = fmod(90*t,360);
   //  Request display update
   glutPostRedisplay();
}

/*
 *  Hellow world main
 */
int main(int argc,char* argv[])
{
   //  Initialize GLUT
   glutInit(&argc,argv);
   //  Request double buffered true color window window with Z-buffer
   glutInitDisplayMode(GLUT_RGB | GLUT_DEPTH | GLUT_DOUBLE);
   //  Create window
   glutCreateWindow("Hello World");
   //  Post callbacks
   glutDisplayFunc(display);
   glutReshapeFunc(reshape);
   glutIdleFunc(idle);
   glutSpecialFunc(special);
   //  Enable Z-buffer depth test
   glEnable(GL_DEPTH_TEST);
   //  Pass control to GLUT to process events
   glutMainLoop();
   //  Exit
   return 0;
}
